
// Decodes modes of the disassembler, 16 bits or 32 bits.
typedef enum {Decode16Bits = 0, Decode32Bits = 1} _DecodeType;

// Static size of strings. Do not change this value.
#define MAX_TEXT_SIZE (60)
typedef struct {
	unsigned int length;
	unsigned char p[MAX_TEXT_SIZE]; // p is a null terminated string.
} _WString;

// This structure holds all information the disassembler generates per instruction.
typedef struct {
	_WString mnemonic; // Mnemonic of decoded instruction, prefixed if required by REP, LOCK etc.
	_WString operands; // Operands of the decoded instruction, up to 3 operands, comma-seperated.
	_WString instructionHex; // Hex dump - little endian, including prefixes.
	unsigned int size; // Size of decoded instruction.
	unsigned long offset; // Start offset of the decoded instruction.
} _DecodedInst;

typedef struct _DINode_ {
	unsigned long size, pos;
	_DecodedInst* di;
	struct _DINode_* next;
} _DINode;

// Return code of the decoding function.
typedef enum {DECRES_SUCCESS, DECRES_MEMORYERR, DECRES_INPUTERR} _DecodeResult;

// distorm_decode //////////////////
// Input:
//         head - A reference to a pointer for the linked list.
//         offset - Origin of the given code.
//         code - Pointer to the code buffer to be disassembled.
//         length - Amount of bytes that should be decoded from the code buffer.
//         dt - Decoding mode, 16 bits (Decode16Bits) or 32 bits (Decode32Bits).
// Output: head will point to the first node of the disassembled instructions linked list.
// Return: DECRES_SUCCESS on success, DECRES_INPUTERR on input error (null code buffer, other decoding mode),
//         DECRES_MEMORYERR when no more memory allocations could be done.
// Side-Effects: Even if the return code is DECRES_MEMORYERR, there might STILL be data in the
//               linked list, thus it should be scanned or disposed by iterating it with ll_get.
///////////////////////////////////

_DecodeResult distorm_decode(_DINode** head, unsigned long offset, unsigned char* code, long length, _DecodeType dt);

// ll_get //
// Input: A reference to a pointer which is the linked list head.
// Output: None.
// Return: NULL when there is no more data to return, otherwise -
//         a pointer to _DecodedInst structure filled with information about the decoded instruction.
//         This pointer should NOT be disposed, a copy can be done for storing the data.
// Side-Effects: 1) The list can be walked only once.
//               2) Scanned nodes are disposed automatically.
///////////
_DecodedInst* ll_get(_DINode** head);